package mcmultipart.client.multipart;

import java.lang.ref.WeakReference;

import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.particle.IParticleFactory;
import net.minecraft.client.particle.Particle;
import net.minecraft.client.particle.ParticleDigging;
import net.minecraft.client.particle.ParticleManager;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.entity.Entity;
import net.minecraft.init.Blocks;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public class AdvancedParticleManager extends ParticleManager {

    private static AdvancedParticleManager instance;

    public static AdvancedParticleManager getInstance(ParticleManager particleManager) {

        if (instance == null) return instance = new AdvancedParticleManager(Minecraft.func_71410_x().field_71441_e,
                Minecraft.func_71410_x().field_71446_o, particleManager);
        instance.field_78878_a = Minecraft.func_71410_x().field_71441_e;
        instance.parent = new WeakReference<ParticleManager>(particleManager);
        return instance;
    }

    private WeakReference<ParticleManager> parent;

    private AdvancedParticleManager(World worldIn, TextureManager rendererIn, ParticleManager parent) {

        super(worldIn, rendererIn);
        this.parent = new WeakReference<ParticleManager>(parent);
    }

    @Override
    public void func_180533_a(BlockPos pos, IBlockState state) {

        ParticleManager p = parent.get();
        if (p != null) p.func_180533_a(pos, state);
    }

    public void addBlockDestroyEffects(BlockPos pos, TextureAtlasSprite icon) {

        int i = 4;
        for (int j = 0; j < i; ++j) {
            for (int k = 0; k < i; ++k) {
                for (int l = 0; l < i; ++l) {
                    double d0 = pos.func_177958_n() + (j + 0.5D) / i;
                    double d1 = pos.func_177956_o() + (k + 0.5D) / i;
                    double d2 = pos.func_177952_p() + (l + 0.5D) / i;
                    this.func_78873_a(new AdvancedEntityDiggingFX(this.field_78878_a, d0, d1, d2, d0 - pos.func_177958_n() - 0.5D, d1 - pos.func_177956_o() - 0.5D,
                            d2 - pos.func_177952_p() - 0.5D, icon).func_174846_a(pos));
                }
            }
        }
    }

    @Override
    public void func_180532_a(BlockPos pos, EnumFacing side) {

        ParticleManager p = parent.get();
        if (p != null) p.func_180532_a(pos, side);
    }

    public void addBlockHitEffects(BlockPos pos, EnumFacing side, AxisAlignedBB box, TextureAtlasSprite icon) {

        int i = pos.func_177958_n();
        int j = pos.func_177956_o();
        int k = pos.func_177952_p();
        float f = 0.1F;
        double d0 = i + this.field_78878_a.field_73012_v.nextDouble() * (box.field_72336_d - box.field_72340_a - f * 2.0F) + f + box.field_72340_a;
        double d1 = j + this.field_78878_a.field_73012_v.nextDouble() * (box.field_72337_e - box.field_72337_e - f * 2.0F) + f + box.field_72338_b;
        double d2 = k + this.field_78878_a.field_73012_v.nextDouble() * (box.field_72334_f - box.field_72339_c - f * 2.0F) + f + box.field_72339_c;

        if (side == EnumFacing.DOWN) d1 = j + box.field_72338_b - f;
        if (side == EnumFacing.UP) d1 = j + box.field_72337_e + f;
        if (side == EnumFacing.NORTH) d2 = k + box.field_72339_c - f;
        if (side == EnumFacing.SOUTH) d2 = k + box.field_72334_f + f;
        if (side == EnumFacing.WEST) d0 = i + box.field_72340_a - f;
        if (side == EnumFacing.EAST) d0 = i + box.field_72336_d + f;

        this.func_78873_a((new AdvancedEntityDiggingFX(this.field_78878_a, d0, d1, d2, 0.0D, 0.0D, 0.0D, icon)).func_174846_a(pos)
                .func_70543_e(0.2F).func_70541_f(0.6F));
    }

    @Override
    public void addBlockHitEffects(BlockPos pos, RayTraceResult target) {

        ParticleManager p = parent.get();
        if (p != null) p.addBlockHitEffects(pos, target);
    }

    public void addBlockHitEffects(BlockPos pos, RayTraceResult target, AxisAlignedBB box, TextureAtlasSprite icon) {

        addBlockHitEffects(pos, target.field_178784_b, box, icon);
    }

    @Override
    public void func_78873_a(Particle effect) {

        ParticleManager p = parent.get();
        if (p != null) p.func_78873_a(effect);
    }

    @Override
    public void func_78870_a(World worldIn) {

        ParticleManager p = parent.get();
        if (p != null) p.func_78870_a(worldIn);
    }

    @Override
    public void func_178926_a(Entity entityIn, EnumParticleTypes particleTypes) {

        ParticleManager p = parent.get();
        if (p != null) p.func_178926_a(entityIn, particleTypes);
    }

    @Override
    public String func_78869_b() {

        if (parent.get() != null) return parent.get().func_78869_b();
        return null;
    }

    @Override
    public void func_178929_a(int id, IParticleFactory particleFactory) {

    }

    @Override
    public void func_78872_b(Entity entityIn, float p_78872_2_) {

        ParticleManager p = parent.get();
        if (p != null) p.func_78872_b(entityIn, p_78872_2_);
    }

    @Override
    public void func_78874_a(Entity entityIn, float partialTicks) {

        ParticleManager p = parent.get();
        if (p != null) p.func_78874_a(entityIn, partialTicks);
    }

    @Override
    public Particle func_178927_a(int particleId, double p_178927_2_, double p_178927_4_, double p_178927_6_, double p_178927_8_,
            double p_178927_10_, double p_178927_12_, int... p_178927_14_) {

        if (parent.get() != null) return parent.get().func_178927_a(particleId, p_178927_2_, p_178927_4_, p_178927_6_, p_178927_8_,
                p_178927_10_, p_178927_12_, p_178927_14_);
        return null;
    }

    @Override
    public void func_78868_a() {

        ParticleManager p = parent.get();
        if (p != null) p.func_78868_a();
    }

    public static class AdvancedEntityDiggingFX extends ParticleDigging {

        protected AdvancedEntityDiggingFX(World worldIn, double xCoordIn, double yCoordIn, double zCoordIn, double xSpeedIn,
                double ySpeedIn, double zSpeedIn, TextureAtlasSprite icon) {

            super(worldIn, xCoordIn, yCoordIn, zCoordIn, xSpeedIn, ySpeedIn, zSpeedIn, Blocks.field_150348_b.func_176223_P());
            func_187117_a(icon);
        }

    }

}
